/*
 * Copyright (C) 2010 Teleal GmbH, Switzerland
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.teleal.cling.test.ssdp;

import org.teleal.cling.model.Constants;
import org.teleal.cling.model.meta.LocalDevice;
import org.teleal.cling.model.message.IncomingDatagramMessage;
import org.teleal.cling.model.message.UpnpResponse;
import org.teleal.cling.model.message.discovery.IncomingSearchResponse;
import org.teleal.cling.model.message.header.EXTHeader;
import org.teleal.cling.model.message.header.HostHeader;
import org.teleal.cling.model.message.header.LocationHeader;
import org.teleal.cling.model.message.header.MaxAgeHeader;
import org.teleal.cling.model.message.header.NTSHeader;
import org.teleal.cling.model.message.header.RootDeviceHeader;
import org.teleal.cling.model.message.header.ServerHeader;
import org.teleal.cling.model.message.header.USNRootDeviceHeader;
import org.teleal.cling.model.message.header.UpnpHeader;
import org.teleal.cling.model.types.NotificationSubtype;
import org.teleal.cling.test.data.SampleData;
import org.teleal.common.util.URIUtil;

import java.net.InetAddress;

public class SearchResponseConcurrentTest extends NotifyAliveConcurrentTest {

    protected IncomingDatagramMessage createMessage() {
        try {

            LocalDevice device = SampleData.createLocalDevice();

            IncomingSearchResponse msg = new IncomingSearchResponse(
                    new IncomingDatagramMessage<UpnpResponse>(
                            new UpnpResponse(UpnpResponse.Status.OK),
                            InetAddress.getByName("127.0.0.1"),
                            Constants.UPNP_MULTICAST_PORT,
                            InetAddress.getByName("127.0.0.1")
                    )
            );

            msg.getHeaders().add(UpnpHeader.Type.EXT, new EXTHeader());
            msg.getHeaders().add(UpnpHeader.Type.HOST, new HostHeader(Constants.IPV4_UPNP_MULTICAST_GROUP, Constants.UPNP_MULTICAST_PORT));
            msg.getHeaders().add(UpnpHeader.Type.MAX_AGE, new MaxAgeHeader(2000));
            msg.getHeaders().add(UpnpHeader.Type.LOCATION, new LocationHeader(
                    URIUtil.createAbsoluteURL(SampleData.getLocalBaseURL(), device.getDescriptorURI())
            ));
            msg.getHeaders().add(UpnpHeader.Type.ST, new RootDeviceHeader());
            msg.getHeaders().add(UpnpHeader.Type.NTS, new NTSHeader(NotificationSubtype.ALIVE));
            msg.getHeaders().add(UpnpHeader.Type.SERVER, new ServerHeader());
            msg.getHeaders().add(UpnpHeader.Type.USN, new USNRootDeviceHeader(device.getIdentity().getUdn()));

            return msg;
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }

    }

}
